/**
 * Table por Process Id
 * It is a kind of fixed size Vector with limited capabilities of setting an element.
 *  
 * Author: Nicolás Wolovick (nicolasw@famaf.unc.edu.ar)
 */
public class PidTable<E> {
	private int capacity;
	private int size;
	private E table[];

	/* Extremely weak but useful Representation Invariant */
	private boolean repInvariant() {
		return 0<=this.size && this.size<=this.capacity;
	}
	
	@SuppressWarnings("unchecked")
	PidTable(int capacity) {
		assert 0<capacity : "Null capacity";
		this.capacity = capacity;
		this.size = 0;
		this.table = (E[]) new Object[capacity]; // http://courses.csail.mit.edu/6.170/old-www/2006-Spring/forum/index.php%3Ftopic=324.msg1131.html
		assert this.repInvariant() : "Representation invariant";
	}
	
	public int alloc(E element) {
		assert element!=null : "Null element";
		assert this.repInvariant() : "Representation invariant";
		boolean found = false;
		int index = 0;
		while (!found && index<this.capacity) {
			if (this.table[index]==null) {
				this.table[index] = element;
				found = true;
			} else {
				index++;
			}
		}
		if (found) {
			this.size++;
			assert this.repInvariant() : "Representation invariant";
			return index;
		} else {
			assert this.repInvariant() : "Representation invariant";
			return this.capacity;
		}
	}
	
	public void release(int index) {
		assert 0<=index && index<this.capacity : "Index out of range";
		assert this.table[index]!=null : "Releasing empty index";
		assert this.repInvariant() : "Representation invariant";
		this.table[index] = null;
		this.size--;
		assert this.repInvariant() : "Representation invariant";
	}
	
	public int capacity() {
		return this.capacity;
	} 
	public int size() {
		return this.size;
	}
	
	public boolean isDef(int index) {
		assert 0<=index && index<this.capacity : "Index out of range";
		return this.table[index]!=null;
	}
	
	public E get(int index) {
		assert 0<=index && index<this.capacity : "Index out of range";
		assert this.repInvariant() : "Representation invariant";
		return this.table[index];
	}
}